// Graph.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <iostream>
#include <vector>
#include <map>
#include <string>
#include <list>
#include <stack>
#include <iostream>

#include <boost/heap/fibonacci_heap.hpp>
#include <climits>

using namespace std;

struct Vertex{
	typedef int weight;
	typedef std::pair<weight,Vertex*> edge; // cost of edge, destination Vertex
	typedef	std::vector<edge> Adj;			// adjacent list
	Adj adj;								// adjacent list
	std::string name;						// name of this node
	Vertex(std::string label) : name(label) {};
};

class Graph
{
public:
	typedef std::map<std::string, Vertex *> NodesMap; // <name, Vertex *>
	
private:
	NodesMap nodes;
	unsigned long numNodes;
	unsigned long numEdges;

public:	
	Graph():numNodes(0), numEdges(0) {};
	~Graph() 
	{
		NodesMap::iterator itr=nodes.begin();
		NodesMap::iterator itr_end=nodes.begin();

		for (; itr!=itr_end;++itr)
			delete itr->second;
	}

	void addNode(const string &name)
	{
		NodesMap::iterator itr=nodes.begin();
		if ((itr=nodes.find(name))== nodes.end())
		{
			Vertex *v = new Vertex(name);
			nodes[name]=v;
			numNodes++;
		}
	}

	void addEdge(const string& from, const string& to, Vertex::weight weight)
	{
		Vertex * f, *t;
		NodesMap::iterator it;
		if ((it = nodes.find(from))!=nodes.end())
			f=it->second;
		else 
			return; //  unknown node
		
		if ((it = nodes.find(to))!=nodes.end())
			t=it->second;
		else
			return; //  unknown node

		pair<Vertex::weight, Vertex *> edge = make_pair(weight,t);
		f->adj.push_back(edge);
		numEdges++;
	}

	NodesMap & getNodes() { return nodes; }
	
	unsigned long getNumNodes() const { return numNodes; }
	unsigned long getNumEdges() const { return numEdges; }
};

std::ostream& operator << (ostream & os,  Graph & g)
{
	Graph::NodesMap::iterator itr=g.getNodes().begin();
	Graph::NodesMap::iterator itr_end=g.getNodes().end();
	
	for(; itr != itr_end; ++itr)
	{
		os << itr->first << ' ' << std::endl;
		Vertex::Adj::const_iterator itrAdj = itr->second->adj.begin();
		Vertex::Adj::const_iterator itrAdjEnd = itr->second->adj.end();
		for (; itrAdj != itrAdjEnd; itrAdj++)
		{
			os << "\t-->" << itrAdj->second->name << " [" << itrAdj->first << "]" << std::endl;
		}
	}
	
	return os;
}

void bfs (Graph & g, std::string s)
{ 
	Vertex * n; std::string name;

	std::list<std::string> queue;        // queue of visited nodes  
	std::map<std::string, bool> visited; // <name, visited>
	Graph::NodesMap nodes = g.getNodes();
	Graph::NodesMap::const_iterator itr =  nodes.begin();  
	Graph::NodesMap::const_iterator itr_end = nodes.end();  

	if (nodes.find(s) == itr_end)       // bfs on unknown node
		return;

	for (; itr != itr_end; ++itr)		
		visited[itr->first]=false;

	visited[s] = true;
	queue.push_back(s);

	std::cout << "Start bfs" << std::endl;

	while (!queue.empty())
	{
		name = queue.front();
		n = nodes[name];
		queue.pop_front();

		std::cout << "Visited: " << name << std::endl;
	
		for (Vertex::Adj::const_iterator itr = n->adj.begin(); itr !=  n->adj.end(); itr++)
		{
			if (!visited[itr->second->name])
			{
				visited[itr->second->name] = true;
				queue.push_back(itr->second->name);
			}
		}
	}
	std::cout << "End bfs" << std::endl << std::endl;
}

void dfsUtil(Graph & g,  std::string & s, std::map<std::string, bool> & visited, 
		 std::map<std::string, unsigned int> & colour, 
		 std::map<std::string, unsigned int> & timing, unsigned int & time)
{
	Graph::NodesMap nodes = g.getNodes();
	Vertex::Adj::const_iterator itr;  
	Vertex::Adj::const_iterator itr_end;    

	visited[s] = true;
	colour[s] = 'g'; // gray
	time++;
	timing[s] = time;

	std::cout << "\t" << s << endl;

	itr =  nodes[s]->adj.begin();
	itr_end =  nodes[s]->adj.end();

	for(; itr!=itr_end; ++itr)
		if (!visited[itr->second->name])
			dfsUtil(g, itr->second->name, visited, colour, timing, time); 

	colour [s] = 'b'; //black
	time++;
}


void dfs (Graph & g, std::string s)
{ 
	Graph::NodesMap nodes = g.getNodes();
	Graph::NodesMap::const_iterator itr =  nodes.begin();  
	Graph::NodesMap::const_iterator itrEnd = nodes.end();  

	std::map<std::string, bool> visited;   
	std::map<std::string, unsigned int> colour;
	std::map<std::string, unsigned int> timing;

	unsigned int time = 0;

	if (nodes.find(s) == itrEnd)       // dfs on unknown node
		return;

	for(; itr!=itrEnd; ++itr)
		visited[itr->second->name] = false;

	for(; itr!=itrEnd; ++itr)
		colour[itr->second->name] = 'w';

	visited[s]=true;

	std::cout << "Starting dfs on " << s << endl;

	Vertex::Adj::const_iterator itrAdj = nodes[s]->adj.begin();
	Vertex::Adj::const_iterator itrAdjEnd = nodes[s]->adj.end();
	
	for (; itrAdj != itrAdjEnd; itrAdj++)
		if (!visited[itrAdj->second->name])
		{
			visited[itrAdj->second->name] = true;
			std::cout << "Starting dfsUtil on " << s << endl;
			dfsUtil(g, itrAdj->second->name, visited, colour, timing, time);			
		}
}


void topologicalSortUtil(Graph & g,  std::string & s, std::map<std::string, bool> & visited, 
		 std::stack<std::string> &stk)
{
	Graph::NodesMap nodes = g.getNodes();
	Vertex::Adj::const_iterator itr;  
	Vertex::Adj::const_iterator itr_end;    

	visited[s] = true;
	
	std::cout << "\t util" << s << endl;

	itr =  nodes[s]->adj.begin();
	itr_end =  nodes[s]->adj.end();

	for(; itr!=itr_end; ++itr)
		if (!visited[itr->second->name])
		topologicalSortUtil(g, itr->second->name, visited, stk); 

	stk.push(s); 
}


void topologicalSort(Graph & g)
{ 
	Graph::NodesMap nodes = g.getNodes();
	Graph::NodesMap::const_iterator itr =  nodes.begin();  
	Graph::NodesMap::const_iterator itrEnd = nodes.end();  

	std::map<std::string, bool> visited;   
	std::stack<std::string> stk;

	for(; itr!=itrEnd; ++itr)
		visited[itr->second->name] = false;

	itr =  nodes.begin();   
	for (; itr != itrEnd; itr++)
		if (!visited[itr->second->name])
			topologicalSortUtil(g, itr->second->name, visited, stk);	

	std::cout << "topologicalSort " << std::endl;
	while(!stk.empty())
	{
		std::cout << stk.top() << std::endl;
		stk.pop();
	}
}


bool isBipartite(Graph & g, std::string s)
{
	Vertex * n; std::string name;
	int numNodes = g.getNumNodes();
	std::map<std::string, int> colours;   

	Graph::NodesMap nodes = g.getNodes();
	Graph::NodesMap::const_iterator itr =  nodes.begin();  
	Graph::NodesMap::const_iterator itrEnd = nodes.end();  

	for(; itr!=itrEnd; ++itr)
		colours[itr->second->name] = -1;

    colours[s] = 1;
 
	std::list<std::string> queue;     
	queue.push_back(s);
 
    while (!queue.empty())
    {
		name = queue.front();
		n = nodes[name];
		queue.pop_front();

		std::cout << "Visited: " << name << std::endl;

		for (Vertex::Adj::const_iterator itr = n->adj.begin(); itr !=  n->adj.end(); itr++)
		{
			if (colours[itr->second->name]==-1) // not yet coloured
			{
				colours[itr->second->name] = 1 - colours[n->name]; // revert colour
				queue.push_back(itr->second->name);
			} else if (colours[itr->second->name] == colours[n->name]) {
                return false;
			}
		}
	}
    return true;
}

class MatrixGraph{

public:
	MatrixGraph(unsigned int numNodes) : n(numNodes)
	{
		M = new int*[n];
		for (unsigned int k = 0 ; k < n; k++)
			M[k]=new int[n];
		for (unsigned int i=0 ; i < n; i++)
			for (unsigned int j=0; j < n; j++)
				M[i][j] = 0;
	}
	~MatrixGraph(){
		for (unsigned int k = 0 ; k < n; k++)
			delete[] M[k];
		delete[] M;
	}

	int & operator() (unsigned int i, unsigned int j) { return M[i][j]; }
	friend std::ostream& operator << (ostream & os, const MatrixGraph & g);
	unsigned int getNumNodes() { return n; }

private:
	unsigned int n;
	int **M;
};

std::ostream& operator << (ostream & os, const MatrixGraph & g)
{
	
	for(unsigned int i=0 ; i < g.n; i++){
		for (unsigned int j=0; j < g.n; j++)
			os << g.M[i][j] << ' ' ;
		os << std::endl;
	}
	return os;
}

struct nodeWeight
{
	unsigned int node;
	unsigned int priority;
	nodeWeight(unsigned int name, unsigned int p) : node(name), priority(p) {}; 

	bool operator < (nodeWeight const & rhs) const  
	{
		return priority > rhs.priority;
	}
};

void PrimMST(MatrixGraph & g, unsigned int startNode)
{
	typedef boost::heap::fibonacci_heap< nodeWeight > Heap; 
	std::map <unsigned int, Heap::handle_type> mapHandles;  // handle for decrease() key
	Heap heap;

	unsigned int numNodes = g.getNumNodes();
	bool * isInHeap = new bool[numNodes];                 // check if this node is still in Heap
	unsigned int * parent = new unsigned int [numNodes+1];

	unsigned int edgeWeight;

	for(unsigned int i=0; i < numNodes;i++)
	{
		boost::heap::fibonacci_heap< nodeWeight >::handle_type handle =
			heap.push( nodeWeight(i, UINT_MAX) ); // infinity weights
		mapHandles[i] = handle;
		isInHeap[i] = true;
	}
	(*(mapHandles[startNode])).priority = 0; // initial node
	heap.increase(mapHandles[startNode]);    // fibonacci has increase ammortized constat
											 // decrease is logaritmic so inverted operator <

	while (!heap.empty()){

		nodeWeight nw = heap.top();  // extract top
		heap.pop();
		isInHeap[nw.node] = false;   // no longer in heap
		
		std::cout << nw.node << ' ' << nw.priority << std::endl;

		for(unsigned int v=0; v < numNodes;v++)
		{
			if ( (edgeWeight = g(nw.node, v) )	// there is an edge (nw.node, v)
				 && (isInHeap[v])				// v is in Heap
				 && ( edgeWeight < (*(mapHandles[v])).priority  ) // improving weight 
				) 
			{
				std::cout << "node=" << v << " with w=" << (*(mapHandles[v])).priority << "]->" << edgeWeight << std::endl;

				(*(mapHandles[v])).priority = edgeWeight;
				 heap.increase(mapHandles[v]);
				 parent[v] = nw.node;
			}
		}
	}

	std::cout << "MST" << std::endl;
	for (unsigned int i=1; i < numNodes; i++)
	{
		std::cout << parent[i] << '-' << i << " weight=" << g(parent[i], i) <<  std::endl;
	}
	delete[] isInHeap;
	delete[] parent;
}


bool isSafe(MatrixGraph  & g, int v, int path[], int pos)
{
    if (!g( path[pos-1], v ))
        return false;
 
    for (int i = 0; i < pos; i++)
        if (path[i] == v)
            return false;
 
    return true;
}

bool hamiltonianCycleUtil(MatrixGraph & g, int path[], int pos)
{
	if (pos == g.getNumNodes())
    {
        if ( g( path[pos-1],  path[0]) )
           return true;
        else
          return false;
    }
 
	for (unsigned int v = 1; v < g.getNumNodes(); v++)
    {
		if (isSafe(g, v, path, pos))
        {
            path[pos] = v;
 
            if (hamiltonianCycleUtil (g, path, pos+1) == true)
                return true;
 
            path[pos] = -1;
        }
    }

    return false;
}

bool hamiltonianCycle(MatrixGraph & g)
{
	unsigned int i;
	unsigned int V = g.getNumNodes();
	int *path = new int[V];
    for (i = 0; i < V; i++)
        path[i] = -1;
 
	bool returnValue;

    path[0] = 0;
    if ( ! hamiltonianCycleUtil(g, path, 1) )
    {
        std::cout << "No Hamiltionian cycle";
        returnValue = false;
    } else {
        std::cout << "Hamiltionian cycle " <<std::endl;

		for (i = 0; i < V; i++)
			std::cout << path[i] << ' ';
		std::cout << std::endl;
		returnValue = true;
	}
	delete [] path;
    return true;
}


int _tmain(int argc, _TCHAR* argv[])
{
	Graph G;
	G.addNode("donald duck");
	G.addNode("daisy duck");
	G.addNode("antonio");
	G.addNode("milena");
	G.addNode("lorena");
	G.addNode("mambo");
	G.addNode("salsa");
	G.addNode("adriana");


	G.addEdge("donald duck", "daisy duck", 69);  // donald duck -> daisy duck
	G.addEdge("donald duck", "antonio", 70);	//   -> antonio  <-
	G.addEdge("daisy duck", "antonio", 71);
	G.addEdge("donald duck", "mambo", 0);
	G.addEdge("donald duck", "salsa", 1);

	G.addEdge("milena", "lorena", 72);
	G.addEdge("lorena", "lorena", 73);
	G.addEdge("lorena", "milena", 74);

	G.addEdge("adriana", "adriano", 72);
	

	std::cout << G;

	bfs (G, "donald duck");
	bfs (G, "lorena");
	bfs (G, "unknown");

	dfs (G, "donald duck");
	
	topologicalSort(G);

	if ( isBipartite(G, "donald duck"))
		std::cout << "Bipartite" <<std::endl;
	else
		std::cout << "non bipartite" << std::endl;

	if ( isBipartite(G, "adriana"))
		std::cout << "bipartite" <<std::endl;
	else
		std::cout << "non bipartite" << std::endl;

	MatrixGraph mgA(5);
	mgA(0, 1) = mgA(1, 2) = mgA(2, 3) = mgA(3, 4) = mgA(4, 0) = 1;
	hamiltonianCycle(mgA);
	std::cout << mgA;

	MatrixGraph mg(9);
	mg(0, 1) = 4;
	mg(0, 7) = 8;
	mg(1, 2) = 8;
	mg(1, 7) = 11;
	mg(2, 3) = 7;
	mg(2, 8) = 2;
	mg(2, 5) = 4;
	mg(3, 4) = 9;
	mg(3, 5) = 14;
	mg(4, 5) = 10;
	mg(5, 6) = 2;
	mg(6, 7) = 1;
	mg(6, 8) = 6;
	mg(7, 8) = 7;
	std::cout << mg;

	PrimMST(mg, 0);

	int i;
	std::cin >> i;
}
	

